function [allids, canlist]=crosslists(canlist, annolists, annopos)
% function [allids, canlist]=crosslists(canlist, annolists, annopos)
% cross annotations for multiple experiments.
% The suggested use of this function is to put all cross annotations into
% canlist. The full id lists of all experiments AND the ids of the gene
% specification should be (column cell) entries in the cell (of cell) 
% array annolists. The column, where we find the cross annotation info
% for a particular experiment (or the gene descriptions) has to be in 
% annopos at the respective position.
% IN
% canlist:    paired lists which define the cross annotations
%             cell array!
% annolists:  all lists that should be reduced and reordered 
%             to the cross annotations.
%             cell array!
% annopos:    annopos(n) is the column index in canlist that
%             corresponds to annolists{:,n}
% 
% OUT
% allids:     positions of the canlist entries in the final cross
%             annotations. allids(:,n) is thus the positions of
%             canlist{annopos(n)}
% canlist:    a reduced cross annotation list. contains only those 
%             entries that were found in all annolists.
%
% (C) P. Sykacke 2006 <peter@sykacek.net>

% sort cross annotation lists
idcan=zeros(size(canlist));
cancols=size(canlist, 2);
sortcan={};
disp('sorting cross annotation list');
for n=1:cancols
  %canlist
  [sortcancol, idcan(:,n)]=my_sort(canlist(:,n));
  sortcan=[sortcan, sortcancol];
end
% sort annolists and cross annotate the sorted lists
%idan=cell(size(annolists));
ancols=size(annolists, 2);
%sortan={};
allcanavail=zeros(size(canlist));
allorigdx=allcanavail;
idallinfull=cell(size(annolists));
for n=1:ancols
  disp(sprintf('cross annotating gene list %1.0f\n', n)); 
  [sortan, idan]=my_sort(annolists{n});
  [idinfull, allcanavail(:,n)]=locann(sortan, sortcan(:,annopos(n)));
  % now convert the positions in the sorted full list to the original positions
  idinfull(find(idinfull))=idan(idinfull(find(idinfull)));
  % and reorder cross annotated ids such that the original positions in the cross \
  % annotations are maintained.
  allorigdx(idcan(find(allcanavail(:,n)),annopos(n)),n)=idinfull(find(idinfull));
  % finally reorder the available cross annotation entries such that we can \
  % reduce them as well 
  allcanavail(idcan(:,annopos(n)),n)=allcanavail(:,n);
end
% we have now allcanavail, which contains the indices of all cross \
% annotations in the respective full lists, ordered w.r.t. the original
% cross annotation list. There are though still some zeros in the
% array (marking missing entries in the full lists) and we have to 
% remove these.
existrows=find(all(allorigdx, 2));
allids=allorigdx(existrows,:);
canexistrows=find(all(allcanavail, 2));
canlist=canlist(canexistrows,:);

function [idinfull, canavail]=locann(fullist, canlist)
% local annotation. assumes fullist and canlist are sorted.
% IN
% fullist:  overall list of strings (sorted)
% canlist:  reduces (cross annotated) list.
% OUT
% idinfull: positions of canlist(idcan) in fullist
% canavail: flags whether canlist(n) is in fullist

nf=1;
nc=1;
lf=length(fullist);
lc=length(canlist);
idinfull=[];
canavail=[];
while nc<=lc
  found=mystrcmp(fullist{nf}, canlist{nc});
  while found < 0 && nf<lf
    nf=nf+1;
    found=mystrcmp(fullist{nf}, canlist{nc});
  end
  if found==0 % then we have it
    idinfull=[idinfull;nf];
    canavail=[canavail; 1];
  else % we do not find one of the cross annotations in the list 
    idinfull=[idinfull;0];
    canavail=[canavail;0];
  end    
  nc=nc+1;
end

function n=mystrcmp(s1, s2)
lmax=min(length(s1), length(s2));
l=1;
while s1(l)==s2(l) && l < lmax
  l=l+1;
end
n=s1(l)-s2(l);
if n==0
  n=length(s1)-length(s2);
end

function [srtd, id]=my_sort(x)
% helper for sorting strings added in 2017.
%disp('my_sort')
%x
is_oct=syschck('oct');
if is_oct && !iscell(x)
  if isstr(ix(1))
    % octave needs a cell array if the erntries are strings
    x=cellstr(x);
    [srtd, id]=sort(x);
  else
    [srtd, id]=sort(x);
  end
else
  %x
  [srtd, id]=sort(x);
end