function [ranktab, id, coregulated]=genranktab(comp, ctr_coexp, vbprb)
% function [ranktab, id, coregulated]=genranktab(comp, ctr_coexp, vbprb)
% this function generates a rank table from comparison.
% we order probabilities, according to the gene measure
% which appart from numerical roundoff is the same as 
% ordering w.r.t subset probabilities.
% the output is a cell array which contains the gene symbol
% and the gene description in the first two columns. The 
% third column is the indicator probabilities, fourth and fifth
% column are the gene measure and the cumulative gene measure.
% IN: 
% comp -                the comparison structure.
% ctr_coexp -           an optional variable that controls how shared function is
%                       handeled.
% if ctr_coexp - 0      no coexpression is checked, genes taken according
%                       to probabilities. This is the default operation
%                       which also happens if we get only the first
%                       parameter.
% if ctr_coexp - 1      coexpression is forced. That is we take only those
%                       genes that are either co up- or co downregulated.
% vbprb -               infered vbprb structure. Used to test coexpression
%                       in systems. Coexpression means that hogher dimensional 
%                       regression coefficients must have the same sign. Otherwise 
%                       the projection slope differs and the genes behave dofferently.  
% Here we need:
% comp.subsP:           subset probabilities (measure accross)         
% comp.indP:            indicator probabilities (measure within)
% comp.genespec, 
% comp.interspec :      gene symbols and gene description and the symbol 
%                       and description used for the empty subset (intercept only prediction).
% OUT:
% ranktab:              ranked gene table with probabilities (see above)
% id:                   original index in genelist (useful for ploting the corresponding fold probabilities)
% coregulated:          boolean list as long as ranktab and flags
%                       coregulation
%
% (C) P. Sykacek. 2005

[cumP, id]=probs2cumprobs(comp.subsP, 1);
%length(id)
cumP=cumP(:);
indP=comp.indP(id);
indP=indP(:);
% add for empty subset
if length(id)>size(comp.genespec, 1)
    ranktab=[comp.genespec; comp.interspec];
else
    ranktab=comp.genespec;
end

subsP=comp.subsP(id);
subsP=subsP(:);
ranktab=ranktab(id,:);
ranktab=[ranktab, num2cell(indP), num2cell(subsP), num2cell(cumP)];

if nargin < 2
    ctr_coexp=0;
end
if (nargin==3) & ctr_coexp % then we reduce the rank table.
    nsys=length(vbprb.sys);
    ntrans=length(vbprb.sys(1).g_mdls)+1; % we have one more transcripts than entries since one is compared against.
    sigagree=zeros(ntrans, 1);
    nrefc=vbprb.refsubspos;
    for n=1:ntrans
        if n < nrefc    % below reference subsetnumber and position in indprobs and subsprobs is the same as subset number
            pos=n;
            ncf=length(vbprb.sys(1).g_mdls(pos).i_mdl(2).beta);
            if ncf > 1
                thissig=sign(vbprb.sys(1).g_mdls(pos).i_mdl(2).beta(2:ncf)); % the slopes only
                alltrue=ones(size(thissig));
                for m=2:nsys
                    alltrue=alltrue&(thissig==sign(vbprb.sys(m).g_mdls(pos).i_mdl(2).beta(2:ncf)));
                end
            else
                alltrue=1;
            end
        elseif n==nrefc % equal to reference subset. position is still n but the coefficients are ar i_mdl(1). (any transcript index!)
            pos=1;
            ncf=length(vbprb.sys(1).g_mdls(pos).i_mdl(1).beta);
            if ncf > 1
                thissig=sign(vbprb.sys(1).g_mdls(pos).i_mdl(1).beta(2:ncf)); % the slopes only
                alltrue=ones(size(thissig));
                for m=2:nsys
                    alltrue=alltrue&(thissig==sign(vbprb.sys(m).g_mdls(pos).i_mdl(1).beta(2:ncf)));
                end
            else
                alltrue=1;
            end
        else % larger than reference subset: pos in indprobs is one abopve subset number!
            pos=n-1;
            ncf=length(vbprb.sys(1).g_mdls(pos).i_mdl(2).beta);
            if ncf > 1
                thissig=sign(vbprb.sys(1).g_mdls(pos).i_mdl(2).beta(2:ncf)); % the slopes only
                alltrue=ones(size(thissig));
                for m=2:nsys
                    alltrue=alltrue&(thissig==sign(vbprb.sys(m).g_mdls(pos).i_mdl(2).beta(2:ncf)));
                end
            else
                alltrue=1;
            end
        end
        sigagree(n)=all(alltrue);
    end
    % reoprder sigagree to get coregulated
    coregulated=sigagree(id);
else
    coregulated=[];
end