function cdef=newcrossan(cdef)
% function allcrossan(cdef)
% cross annotates a series of datasets to a target genome.
% cdef.cfname:      file name with cross annotation information.
% cdef.nspec:       number of species (i.e. columns) in the cross annotation file.
% cdef.spectype:    species type (first row in cross annotation file)
% cdef.crossanid:   stored cross annotation id.
% cdef.logfile:     log filename where cdef is stored.
% cdef.targspecdx:  column index of target species in the cross annotation file name
% cdef.gspcifnam:   input file name of gene specification, genespec which is a {n x 3} 
%                   cell array of gene specifications. This is a cell array that
%                   contains in the first column a geneid's, in the second column a short
%                   gene symbol and in the third column an extended gene description.
% cdef.gspcofnam:   output file name of gene specification. 
%                   this file is later used in combres2figtab as input for getting
%                   gene symbols and gene descriptions.
% cdef.genedesc:    reduced and reordered gene specificstion.
% cdef.donormalise: if true - we remove the mean and the std. deviation of every variable in the
%                   dataset (i.e. we make all log expressions or loig ratios zero mean and unit 
%                   std. dev.)
% cdef.exps:        all experiments that should be cross annotated.
% exps(n).mcafname: file name with fspma outputed (and possibly normalised) microarray data.
% exps(n).trgfname: file name with fspma outputed sample classifications. or, if not a string
%                   class labels provided directly.
% exps(n).trandx:   training indices in data. if all data is for training we use nan. if trandx is not nan
%                   tstdx must not be either.
% exps(n).tstdx:    test indices in data.
% exps(n).srcspec:  index of source species in cross annotation information.
% exps(n).datafnam: output matlab filename with subsets, geneids and train and/or test file
% 
% (C) P. Sykacek 2005, 2006


% first we read the cross annotation information
strgfrm=repmat('%s ', 1, cdef.nspec);
%strgfrm=strgfrm(:, ones(1, cdef.nspec));
%strgfrm=strgfrm(:)';
disp('load cross annotation information');
[hdr, data]=read_tabtxt(cdef.cfname, strgfrm, '\n\t');
%hdr
%data
cdef.spectype=hdr;
ncol=length(data);
cdef.crossanid={};
for n=1:ncol
    cdef.crossanid=[cdef.crossanid,data{n}];
end
% now loop over (biological) experiments and cross annotate data.
nexp=length(cdef.exps);
n=1;
allcpos=[];
while n<=nexp
    disp(sprintf('load data: %s\n', cdef.exps(n).mcafname));
    % load data
    [genes, data]=scan_fspma(cdef.exps(n).mcafname);
    % for test purpouse: reduce to the first 1000 entries.
    %disp('testing: reduced size line 53!');
    %genes=genes(1:1000);
    %data=data(:,1:1000);
    if cdef.donormalise
        mdt=mean(data);
        data=data-mdt(ones(size(data, 1), 1),:);
        sdt=std(data);
        data=data./sdt(ones(size(data, 1), 1),:);
    end
    % check trgfname and read it from the effects file (or take it as classifications).
    if ischar(cdef.exps(n).trgfname)
        [header, classlabels]=scan_fspma(cdef.exps(n).trgfname);
        targets{n}=classlabels(:,1);
    else
        targets{n}=cdef.exps(n).trgfname;
    end
    alldata{n}=data;
    allgenlists{n}=genes(:);
    disp(size(allgenlists{n}));
    allcpos=[allcpos, cdef.exps(n).srcspec];
    n=n+1;
end
% load gene specification
% new: read gene description from csv file.
disp('load gene specification from csv file.');
[genedesc, header]=read_affyannolist(cdef.gspcifnam, cdef.gspccols);
%disp('testing: reduced size line 77!');
%genedesc=genedesc(1:1000,:);
%alldata{n}=genedesc;

if syschck('mlb')
    sfmode='-v6'; % MatLab
else
    sfmode='-mat'; % Octave
end

allgenlists{n}=genedesc(:,1);
allcpos=[allcpos, cdef.targspecdx];
disp('start cross annotation');
%cdef.crossanid
%allgenlists
%allcpos
[allids, cdef.reducanlist]=crosslists(cdef.crossanid, allgenlists, allcpos);
cdef.allids=allids;
cdef.allgenlists=allgenlists;
n=1;
while n<=nexp
    %reducing the data: 
    ndata=alldata{n};
    ndata=[ndata(:, allids(:, n)'), targets{n}];
    geneids=cdef.reducanlist(:, cdef.targspecdx);
    if any(isnan(cdef.exps(n).trandx))
       train=ndata;
       test=[];
    else
       train=ndata(cdef.exps(n).trandx, :);
       test=ndata(cdef.exps(n).tstdx,:);
    end
    subsets=num2cell([1:length(geneids)]);
    % save cross annotated data
    if ~isempty(test)
        save(cdef.exps(n).datafnam, sfmode, 'subsets', 'train', 'geneids', 'test');
    else
        save(cdef.exps(n).datafnam, sfmode, 'subsets', 'train', 'geneids');
    end
    n=n+1;
end
% finally we generate and write the gene specification.
genedesc=genedesc(allids(:,n),:);
[genedesc]=simpleaffygenesym(genedesc, '///');		 
cdef.genedesc=genedesc;
% and save new gene descriptions
save(cdef.gspcofnam, sfmode, 'genedesc');
% save log file.
save(cdef.logfile, sfmode, 'cdef');